/*
 * Copyright (c) 2016 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mps2_buttons.h"
#include "smm_mps2.h"
#include "platform_retarget_dev.h"
#include "platform_retarget_pins.h"

#define MPS2_FPGAIO_NBR_BTN 2U  /* Number of available MPS2 FPGAIO buttons */
#define MPS2_SHIELD_NBR_BTN 2U  /* Number of available MPS2 SHIELD buttons */
#define MPS2_SCC_NBR_SW     8U  /* Number of available MPS2 SCC buttons */

#define SHIELD_PB_INITALIZED 1
#define SHIELD_PB_NOT_INITALIZED 0
static uint32_t shield_pb_init = SHIELD_PB_NOT_INITALIZED;

/* Local function */
static void mps2_shield_btns_init(void);

uint32_t mps2_get_nbr_btns(enum mps2_button_t btn_type)
{
    switch(btn_type) {
    case MPS2_FPGAIO_BTN:
        return MPS2_FPGAIO_NBR_BTN;
    case MPS2_SHIELD_BTN:
        return MPS2_SHIELD_NBR_BTN;
    case MPS2_SCC_SW:
        return MPS2_SCC_NBR_SW;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all enumeration values are
     *           covered in the switch.*/
    }
}

uint32_t mps2_get_all_btns_state(enum mps2_button_t btn_type)
{
    uint32_t but_val;

    switch(btn_type) {
    case MPS2_FPGAIO_BTN:
        but_val = MPS2_FPGAIO->BUTTON;
        but_val &= ((1U << (MPS2_FPGAIO_NBR_BTN)) - 1U);
        break;
    case MPS2_SHIELD_BTN:
        if (shield_pb_init == SHIELD_PB_NOT_INITALIZED) {
            mps2_shield_btns_init();
        }
        but_val = (uint32_t)
            arm_gpio_read(&ARM_GPIO1_DEV_NS, ARM_GPIO_ACCESS_PORT, 0);
        /*
         * Put the two buttons in the first 2 bits.
         */
        but_val >>= SHIELD_ADAPTOR_PB0;
        but_val &= ((1U << (MPS2_SHIELD_NBR_BTN)) - 1U);
        break;
    case MPS2_SCC_SW:
        but_val = MPS2_SCC->SWITCHES;
        but_val &= ((1U << (MPS2_SCC_NBR_SW)) - 1U);
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all enumeration values are
     *           covered in the switch.*/
    }

    return but_val;
}

uint32_t mps2_get_btn_state(enum mps2_btn_name_t btn_id)
{
    const volatile uint32_t* p_reg;

    switch(btn_id) {
    case MPS2_FPGAIO_USERB0:
    case MPS2_FPGAIO_USERB1:
        p_reg = &MPS2_FPGAIO->BUTTON;
        break;
    case MPS2_SHIELD_USERB0:
    case MPS2_SHIELD_USERB1:
        if (shield_pb_init == SHIELD_PB_NOT_INITALIZED) {
            mps2_shield_btns_init();
        }
        return (uint32_t)arm_gpio_read(&ARM_GPIO1_DEV_NS, ARM_GPIO_ACCESS_PIN,
                                       btn_id == MPS2_SHIELD_USERB0 ?
                                       SHIELD_ADAPTOR_PB0 : SHIELD_ADAPTOR_PB1);
    case MPS2_SCC_SW1:
    case MPS2_SCC_SW2:
    case MPS2_SCC_SW3:
    case MPS2_SCC_SW4:
    case MPS2_SCC_SW5:
    case MPS2_SCC_SW6:
    case MPS2_SCC_SW7:
    case MPS2_SCC_SW8:
        p_reg = &MPS2_SCC->SWITCHES;
        btn_id -= MPS2_SCC_SW1;
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all enumeration values are
     *           covered in the switch.*/
    }

    return (0x00000001U & (*p_reg >> btn_id));
}

/**
 * \brief Initialize GPIO to use shield buttons instead of FPGA buttons.
*/
static void mps2_shield_btns_init(void)
{
    arm_gpio_init(&ARM_GPIO1_DEV_NS);
    (void)arm_gpio_config(&ARM_GPIO1_DEV_NS, ARM_GPIO_ACCESS_PIN,
                          SHIELD_ADAPTOR_PB0,
                          ARM_GPIO_PIN_ENABLE | ARM_GPIO_INPUT);
    (void)arm_gpio_config(&ARM_GPIO1_DEV_NS, ARM_GPIO_ACCESS_PIN,
                          SHIELD_ADAPTOR_PB1,
                          ARM_GPIO_PIN_ENABLE | ARM_GPIO_INPUT);

    shield_pb_init = SHIELD_PB_INITALIZED;
}
