/*
 * Copyright (c) 2016 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __MPS2_GLCD_H__
#define __MPS2_GLCD_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdbool.h>

/* mps2_glcd RGB color definitions */
#define MPS2_GLCD_BLACK       0x0000  /*   0,   0,   0 */
#define MPS2_GLCD_NAVY        0x000F  /*   0,   0, 128 */
#define MPS2_GLCD_DARK_GREEN  0x03E0  /*   0, 128,   0 */
#define MPS2_GLCD_DARK_CYAN   0x03EF  /*   0, 128, 128 */
#define MPS2_GLCD_BROWN       0x7800  /* 128,   0,   0 */
#define MPS2_GLCD_PURPLE      0x780F  /* 128,   0, 128 */
#define MPS2_GLCD_OLIVE       0x7BE0  /* 128, 128,   0 */
#define MPS2_GLCD_LIGHT_GREY  0xC618  /* 192, 192, 192 */
#define MPS2_GLCD_DARK_GREY   0x7BEF  /* 128, 128, 128 */
#define MPS2_GLCD_BLUE        0x001F  /*   0,   0, 255 */
#define MPS2_GLCD_GREEN       0x07E0  /*   0, 255,   0 */
#define MPS2_GLCD_CYAN        0x07FF  /*   0, 255, 255 */
#define MPS2_GLCD_RED         0xF800  /* 255,   0,   0 */
#define MPS2_GLCD_MAGENTA     0xF81F  /* 255,   0, 255 */
#define MPS2_GLCD_YELLOW      0xFFE0  /* 255, 255, 0   */
#define MPS2_GLCD_WHITE       0xFFFF  /* 255, 255, 255 */

/* Screen Size */
#define MPS2_GLCD_WIDTH  320  /* Screen Width (in pixels) */
#define MPS2_GLCD_HEIGHT 240  /* Screen Hight (in pixels) */

/* Font size */
enum mps2_glcd_font_size_t {
    MPS2_GLCD_FONT_SIZE_6x8,    /*!< Font size  6x8 */
    MPS2_GLCD_FONT_SIZE_16x24,  /*!< Font size  16x24 */
};

enum mps2_glcd_error_t {
    MPS2_GLCD_ERR_NONE,         /*!< No error */
    MPS2_GLCD_ERR_INVALID_ARG,  /*!< Invalid input argument */
};

/**
 * \brief Initializes Graphical LCD.
 */
void mps2_glcd_init(void);

/**
 * \brief Sets draw window region.
 *
 * \param[in] x  Horizontal position.
 * \param[in] y  Vertical position.
 * \param[in] w  Window width in pixel.
 * \param[in] h  Window height in pixels.
 */
void mps2_glcd_set_win(uint16_t x, uint16_t y, uint16_t w, uint16_t h);

/**
 * \brief Sets landscape.
 *
 * \param[in] landscape  Landscape configuration (True landscape).
 */
void mps2_set_landscape(bool landscape);

/**
 * \brief Rotates screen 180 degrees.
 *
 * \param[in] rotate  Rotate configuration (True rotate).
 */
void mps2_set_rotate180(bool rotate);

/**
 * \brief Draws a pixel in foreground color.
 *
 * \param[in] x  Horizontal position.
 * \param[in] y  vertical position.
 */
void mps2_glcd_put_pixel(uint16_t x, uint16_t y);

/**
 * \brief Draws a pixel in supplied color.
 *
 * \param[in] x      Horizontal position.
 * \param[in] y      Vertical position.
 * \param[in] color  Color of pixel.
 */
void mps2_glcd_put_pixel_color(uint16_t x, uint16_t y, uint16_t color);

/**
 * \brief Sets foreground color.
 *
 * \param[in] color  Foreground color.
 */
void mps2_glcd_set_text_color(uint16_t color);

/**
 * \brief Sets background color.
 *
 * \param[in] color  Background color.
 */
void mps2_glcd_set_back_color(uint16_t color);

/**
 * \brief Clears display.
 *
 * \param[in] color  Display clearing color.
 */
void mps2_glcd_clear(uint16_t color);

/**
 * \brief Draws character on given position.
 *
 * \param[in] x   Horizontal position.
 * \param[in] y   Vertical position.
 * \param[in] cw  Character width in pixel.
 * \param[in] ch  Character height in pixels.
 * \param[in] c   Pointer to character bitmap.
 */
void mps2_glcd_draw_char(uint16_t x, uint16_t y, uint16_t cw,
                         uint16_t ch, const uint8_t* c);

/**
 * \brief Displays character on given line.
 *
 * \param[in] ln    Line number.
 * \param[in] col   Column number.
 * \param[in] font  Font size as specified in \ref mps2_glcd_font_size_t
 * \param[in] c     ASCII character.
 */
void mps2_glcd_display_char(uint16_t ln, uint16_t col,
                            enum mps2_glcd_font_size_t font, uint8_t c);

/**
 * \brief Displays string on given line.
 *
 * \param[in] ln     Line number.
 * \param[in] col    Column number.
 * \param[in] font   Font size as specified in \ref mps2_glcd_font_size_t
 * \param[in] p_str  Pointer to string to display.
 *
 * \return Returns error code as specified in \ref mps2_glcd_error_t
 */
enum mps2_glcd_error_t mps2_glcd_display_string(uint16_t ln, uint16_t col,
                                                enum mps2_glcd_font_size_t font,
                                                const char* p_str);

/**
 * \brief Clears the given line.
 *
 * \param[in] ln    Line number.
 * \param[in] font  Font size as specified in \ref mps2_glcd_font_size_t
 */
void mps2_glcd_clear_ln(uint16_t ln, enum mps2_glcd_font_size_t font);

/**
 * \brief Draws bargraph.
 *
 * \param[in] x    Horizontal position.
 * \param[in] y    Vertical position.
 * \param[in] w    Maximum width of bargraph (in pixels).
 * \param[in] h    Bargraph height.
 * \param[in] val  Value of active bargraph (in 1/1024).
 */
void mps2_glcd_bargraph(uint16_t x, uint16_t y, uint16_t w,
                        uint16_t h, uint16_t val);

/**
 * \brief Displays a graphical bitmap image at position x horizontally
 *        and y vertically (This function is optimized for 16 bits per
 *        pixel format, it has to be adapted for any other bits per
 *        pixel format).
 *
 * \param[in] x       Horizontal position.
 * \param[in] y       Vertical position.
 * \param[in] w       Maximum width of bargraph (in pixels).
 * \param[in] h       Bargraph height.
 * \param[in] bitmap  Address at which the bitmap data resides.
 *
 * \return Returns error code as specified in \ref mps2_glcd_error_t
 */
enum mps2_glcd_error_t mps2_glcd_bitmap(uint16_t x, uint16_t y, uint16_t w,
                                        uint16_t h, uint16_t* bitmap);

/**
 * \brief Scrolls content of the whole display for dy pixels vertically.
 *
 * \param[in] dy  Horizontal position.
 */
void mps2_glcd_scroll_vertical(uint16_t dy);

/**
 * \brief Draws box filled with color.
 *
 * \param[in] x      Horizontal position.
 * \param[in] y      Vertical position.
 * \param[in] w      Window width in pixels.
 * \param[in] h      Window height in pixels.
 * \param[in] color  Color of pixel.
 */
void mps2_glcd_box(uint16_t x, uint16_t y, uint16_t w,
                   uint16_t h, uint16_t color);

/**
 * \brief Draws box boarded with color.
 *
 * \param[in] x      Horizontal position.
 * \param[in] y      Vertical position.
 * \param[in] w      Window width in pixels.
 * \param[in] h      Window height in pixels.
 * \param[in] color  Color of pixel.
 */
void mps2_glcd_box_border(uint16_t x, uint16_t y, uint16_t w,
                          uint16_t h, uint16_t color);

#ifdef __cplusplus
}
#endif
#endif /* __MPS2_GLCD_H__ */
