/*
 * Copyright (c) 2016 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * nless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mps2_touchscreen.h"

#include "Driver_I2C.h"
#include "mps2_time.h"

#define TSC_I2C_ADDR  0x82  /* 7-bit I2C address */

/* I2C Driver */
extern ARM_DRIVER_I2C Driver_I2C0;
static ARM_DRIVER_I2C* I2C_DRV = &Driver_I2C0;

/* STMPE811 register addresses */
enum stmpe811_reg_t {
    STMPE811_CHIP_ID        = 0x00,
    STMPE811_ID_VER         = 0x02,
    STMPE811_SYS_CTRL1      = 0x03,
    STMPE811_SYS_CTRL2      = 0x04,
    STMPE811_SPI_CFG        = 0x08,
    STMPE811_INT_CTRL       = 0x09,
    STMPE811_INT_EN         = 0x0A,
    STMPE811_INT_STA        = 0x0B,
    STMPE811_GPIO_EN        = 0x0C,
    STMPE811_GPIO_INT_STA   = 0x0D,
    STMPE811_ADC_INT_EN     = 0x0E,
    STMPE811_ADC_INT_STA    = 0x0F,
    STMPE811_GPIO_SET_PIN   = 0x10,
    STMPE811_GPIO_CLR_PIN   = 0x11,
    STMPE811_GPIO_MP_STA    = 0x12,
    STMPE811_GPIO_DIR       = 0x13,
    STMPE811_GPIO_ED        = 0x14,
    STMPE811_GPIO_RE        = 0x15,
    STMPE811_GPIO_FE        = 0x16,
    STMPE811_GPIO_AF        = 0x17,
    STMPE811_ADC_CTRL1      = 0x20,
    STMPE811_ADC_CTRL2      = 0x21,
    STMPE811_ADC_CAPT       = 0x22,
    STMPE811_ADC_DATA_CH0   = 0x30,
    STMPE811_ADC_DATA_CH1   = 0x32,
    STMPE811_ADC_DATA_CH2   = 0x34,
    STMPE811_ADC_DATA_CH3   = 0x36,
    STMPE811_ADC_DATA_CH4   = 0x38,
    STMPE811_ADC_DATA_CH5   = 0x3A,
    STMPE811_ADC_DATA_CH6   = 0x3C,
    STMPE811_ADC_DATA_CH7   = 0x3E,
    STMPE811_TSC_CTRL       = 0x40,
    STMPE811_TSC_CFG        = 0x41,
    STMPE811_WDW_TR_X       = 0x42,
    STMPE811_WDW_TR_Y       = 0x44,
    STMPE811_WDW_BL_X       = 0x46,
    STMPE811_WDW_BL_Y       = 0x48,
    STMPE811_FIFO_TH        = 0x4A,
    STMPE811_FIFO_STA       = 0x4B,
    STMPE811_FIFO_SIZE      = 0x4C,
    STMPE811_TSC_DATA_X     = 0x4D,
    STMPE811_TSC_DATA_Y     = 0x4F,
    STMPE811_TSC_DATA_Z     = 0x51,
    STMPE811_TSC_FRACTION_Z = 0x56,
    STMPE811_TSC_DATA_XYZ   = 0x57,
    STMPE811_TSC_DATA       = 0xD7,
    STMPE811_TSC_I_DRIVE    = 0x58,
    STMPE811_TSC_SHIELD     = 0x59,
    STMPE811_TEMP_CTRL      = 0x60,
    STMPE811_TEMP_DATA      = 0x61,
    STMPE811_TEMP_TH        = 0x62,
};

/*
 * \brief Reads data from touchscreen controller
 *
 * \param[in] reg  Register to read (\ref stmpe811_reg_t).
 * \param[in] val  Pointer to read data
 *
 * \returns Returns error code as specified in \ref mps2_touch_error_t
 */
static enum mps2_touch_error_t touch_read(enum stmpe811_reg_t reg,
                                          uint8_t *val)
{
    int32_t ret;

    ret = I2C_DRV->MasterTransmit(TSC_I2C_ADDR, (uint8_t*)&reg, 1, true);
    if(ret != ARM_DRIVER_OK) {
        return MPS2_I2C_TX_ERROR;
    }

    ret = I2C_DRV->MasterReceive(TSC_I2C_ADDR, val, 1, false);
    if(ret != ARM_DRIVER_OK) {
        return MPS2_I2C_RX_ERROR;
    }

    return MPS2_TOUCH_ERR_OK;
}

/*
 * \brief Writes data to touchscreen controller
 *
 * \param[in] reg  Register to write (\ref stmpe811_reg_t).
 * \param[in] val  Data to write
 *
 * \returns Returns error code as specified in \ref mps2_touch_error_t
 */
static enum mps2_touch_error_t touch_write(enum stmpe811_reg_t reg,
                                           uint8_t val)
{
    uint8_t data[2];
    int32_t ret;

    data[0] = reg;
    data[1] = val;

    ret = I2C_DRV->MasterTransmit(TSC_I2C_ADDR, data, 2, false);
    if(ret != ARM_DRIVER_OK) {
        return MPS2_I2C_TX_ERROR;
    }

    return MPS2_TOUCH_ERR_OK;
}

enum mps2_touch_error_t mps2_touch_init(void)
{
    int32_t cmsis_ret;
    enum mps2_touch_error_t ret;

    cmsis_ret = I2C_DRV->Initialize(NULL);
    if(cmsis_ret != ARM_DRIVER_OK) {
        return MPS2_I2C_INIT_ERROR;
    }

    /* Reset touchscreen controller */
    ret = touch_write(STMPE811_SYS_CTRL1, 0x02);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }
    mps2_sleepms(10);

    /* Enable TSC and ADC */
    ret = touch_write(STMPE811_SYS_CTRL2, 0x0C);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Enable Touch detect, FIFO */
    ret = touch_write(STMPE811_INT_EN, 0x07);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Set sample time, 12-bit mode */
    ret = touch_write(STMPE811_ADC_CTRL1, 0x69);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }
    mps2_sleepms(10);

    /* ADC frequency 3.25 MHz */
    ret = touch_write(STMPE811_ADC_CTRL2, 0x01);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* 1 sample, detect delay 50ms, settle time 100ms */
    ret = touch_write(STMPE811_TSC_CFG, 0x3F);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Threshold for FIFO */
    ret = touch_write(STMPE811_FIFO_TH, 0x01);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* FIFO reset */
    ret = mps2_touch_reset();
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Drive 50 mA typical */
    ret = touch_write(STMPE811_TSC_I_DRIVE, 0x01);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Pins are used for touchscreen */
    ret = touch_write(STMPE811_GPIO_AF, 0x00);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Enable TSC, TSC operating mode X, Y only */
    ret = touch_write(STMPE811_TSC_CTRL, 0x03);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Clear interrupt status */
    ret = touch_write(STMPE811_INT_STA, 0xFF);

    return ret;
}

enum mps2_touch_error_t mps2_touch_get_state(struct mps2_touch_state_t* p_state)
{
    int32_t cmsis_ret;
    enum mps2_touch_error_t ret;
    uint8_t size;
    uint8_t val;
    uint8_t xy[3];

    if(p_state == 0) {
        return MPS2_INVALID_ARG;
    }

    /* Read touch status */
    ret = touch_read(STMPE811_TSC_CTRL, &val);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    p_state->pressed = (val & (1 << 7)) ? 1 : 0;

    if (p_state->pressed) {
        val = STMPE811_TSC_DATA;

        ret = touch_read(STMPE811_FIFO_SIZE, &size);
        if(ret != MPS2_TOUCH_ERR_OK) {
            return ret;
        }

        if (size == 0) {
            return MPS2_FIFO_EMPTY;
        }

        /* If FIFO overflow, discard all samples except the last one */
        do {
            cmsis_ret = I2C_DRV->MasterTransmit(TSC_I2C_ADDR, &val, 1, true);
            if(cmsis_ret != ARM_DRIVER_OK) {
                return MPS2_I2C_TX_ERROR;
            }
            cmsis_ret = I2C_DRV->MasterReceive(TSC_I2C_ADDR, xy, 3, (size > 1));
            if(cmsis_ret != ARM_DRIVER_OK) {
                return MPS2_I2C_RX_ERROR;
            }
            size--;
        }while(size);

        p_state->x = (int16_t)((xy[0] << 4) | ((xy [1] & 0xF0) >> 4));
        p_state->y = (int16_t)(xy[2] | ((xy[1] & 0x0F) << 8));
    }

    /* FIFO reset */
    ret = mps2_touch_reset();
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    return ret;
}

enum mps2_touch_error_t mps2_touch_reset()
{
    enum mps2_touch_error_t ret;

    ret = touch_write(STMPE811_FIFO_STA,  0x01);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    ret = touch_write(STMPE811_FIFO_STA,  0x00);
    if(ret != MPS2_TOUCH_ERR_OK) {
        return ret;
    }

    /* Clear interrupt flags */
    ret =touch_write(STMPE811_INT_STA, 0x1F);

    return ret;
}
