/*
 * Copyright (c) 2017 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>

#include "cmsis.h"
#include "Driver_MPC.h"
#include "Driver_PPC.h"
#include "mps2_devices.h"
#include "region_defs.h"
#include "s_ns_shared.h"
#include "smm_mps2.h"
#include "mps2_time.h"

/* Clear LSB of the function address to indicate the function-call
   will perform the switch from secure to non-secure */
#define cmse_nsfptr_create(p)   ((__typeof__(p)) ((uint32_t) p & ~(0x1UL)))

/* Avoids the semihosting issue */
#if defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__asm("  .global __ARM_use_no_argv\n");
#endif

/* Defines NS code access to LEDs */
#define FPGA_SCC_LED_BASE  MPS2_SCC_BASE_NS
#define FPGA_SCC_LED_LIMIT (FPGA_SCC_LED_BASE + 0x008)

/* GLCD square area */
#define BOX_W  160
#define BOX_H  40
#define BOX_X  ((MPS2_GLCD_WIDTH / 2) - (BOX_W / 2))
#define BOX_Y  ((MPS2_GLCD_HEIGHT / 2) - (BOX_H / 2))

#define TEXT_START_X  ((BOX_Y+(BOX_H/2))/8)
#define TEXT_START_Y  ((BOX_X+12)/6)

/* Import MPC and PPC drivers */
extern ARM_DRIVER_MPC Driver_SRAM1_MPC;
extern ARM_DRIVER_MPC Driver_SRAM2_MPC;
extern ARM_DRIVER_PPC Driver_APB_PPCEXP2;

static const char glcd_text[] = "Hello from Secure World!";

/*
 * \brief Configures the Memory Protection Controller
 */
void MPC_config(void);

/*
 * \brief Configures the Private Peripheral Controller (PPC)
 */
void PPC_config(void);

/*
 * \brief Enables the fault handlers BusFault, UsageFault,
 *        MemManageFault and SecureFault
 */
void enable_fault_handlers(void);

/*
 * \brief Executes non-secure code
 */
void execute_ns_code(void);

/*
 * \brief     Checks if the user is pressing the touchscreen central area.
 *
 * \parma[in] x   X position
 * \parma[in] y   Y position.
 *
 * \return    True if the user is pressing central area, otherwise false.
 */
bool is_cursor_in_center(int16_t x, int16_t y);

int main(void)
{
    enable_fault_handlers();

    /* Initializes GLCD */
    mps2_glcd_init();
    mps2_glcd_clear(MPS2_GLCD_WHITE);

    /* Draw rectangle in the touch active area */
    mps2_glcd_box_border(BOX_X-1, BOX_Y-1, BOX_W+1, BOX_H+1, MPS2_GLCD_BLUE);
    mps2_glcd_display_string(TEXT_START_X, TEXT_START_Y,
                             MPS2_GLCD_FONT_SIZE_6x8, glcd_text);
    /* Configure resources between secure and non-secure worlds */
    MPC_config();
    PPC_config();

    execute_ns_code();
}

uint32_t get_btns_state(void)
{
    uint32_t btns_state;

    /* If the MPS2 adapter for Arduino (AN502) is plugged in, replace
     * MPS2_FPGAIO_BTN by MPS2_SHIELD_BTN to use the buttons in the
     * adapter instead of USER buttons in the MPS2 board.
     */
    btns_state = ((mps2_get_all_btns_state(MPS2_SCC_SW) << 8) & 0x0000FF00);
    btns_state |= (0x000000FF & mps2_get_all_btns_state(MPS2_FPGAIO_BTN));

    if(btns_state & BT0_MASK) {
        /* User button 0 pressed */
        mps2_led_on(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED0);
    } else {
        mps2_led_off(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED0);
    }

    if (btns_state & BT1_MASK) {
        /* User button 1 pressed */
        mps2_led_on(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED1);
    } else {
        mps2_led_off(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED1);
    }

    return btns_state;
}

void sleep_ms(uint32_t ms)
{
    mps2_sleepms(ms);
}

void execute_ns_code(void)
{
    /* SCB_NS.VTOR points to the Non-secure vector table base address */
    SCB_NS->VTOR = (NS_CODE_START);

    /* Setup Main stack pointer of the non-secure code */
    uint32_t ns_msp = *(uint32_t*)(NS_CODE_START);
    __TZ_set_MSP_NS(ns_msp);

    /* The entry contains adddress of the Reset_handler (CMSIS-CORE) function */
    uint32_t entry_ptr = *(uint32_t*)(NS_CODE_START+4);

    /* Clear LSB of the function address to indicate the function-call
       will perform the switch from secure to non-secure */
    nsfptr_t ns_entry = (nsfptr_t)cmse_nsfptr_create(entry_ptr);

    /* All changes made to memory will be effective after this point */
    __DSB();
    __ISB();

    /* Call the non-secure Reset_Handler to jump to the non-secure binary */
    ns_entry();
}

void enable_fault_handlers(void)
{
    SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk
               |  SCB_SHCSR_BUSFAULTENA_Msk
               |  SCB_SHCSR_MEMFAULTENA_Msk
               |  SCB_SHCSR_SECUREFAULTENA_Msk;
}

void MPC_config(void)
{
    /* Initialize the MPCs */
    Driver_SRAM1_MPC.Initialize();
    Driver_SRAM2_MPC.Initialize();

    /* Configure the MPCs */
    Driver_SRAM1_MPC.ConfigRegion(NS_CODE_START, NS_CODE_LIMIT-1,
                                  ARM_MPC_ATTR_NONSECURE);

    Driver_SRAM2_MPC.ConfigRegion(NS_DATA_START, NS_DATA_LIMIT-1,
                                  ARM_MPC_ATTR_NONSECURE);

    /* Lock down the MPC configurations */
    Driver_SRAM1_MPC.LockDown();
    Driver_SRAM2_MPC.LockDown();

    /* Add barriers to assure the MPC configuration is done before continue
     * the execution. */
    __DSB();
    __ISB();
}

void PPC_config(void)
{
    Driver_APB_PPCEXP2.Initialize();

    /* Configure FPGA_SCC as a nonsecure and non privilage */
    Driver_APB_PPCEXP2.ConfigPeriph(CMSDK_FPGA_SCC_PPC_POS,
                                    ARM_PPC_NONSECURE_ONLY,
                                    ARM_PPC_PRIV_AND_NONPRIV);
}
