/*
 * Copyright (c) 2016 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

#include "cmsis.h"
#include "mps2_devices.h"
#include "mps2_time.h"

/* Avoids the semihosting issue */
#if defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__asm("  .global __ARM_use_no_argv\n");
#endif

/* Touchscreen central area */
#define MIN_X  1545
#define MAX_X  2545
#define MIN_Y  1445
#define MAX_Y  2445

/* GLCD square area */
#define BOX_W  80
#define BOX_H  ((MPS2_GLCD_HEIGHT * BOX_W) / MPS2_GLCD_WIDTH)
#define BOX_X  ((MPS2_GLCD_WIDTH / 2) - (BOX_W / 2))
#define BOX_Y  ((MPS2_GLCD_HEIGHT / 2) - (BOX_H / 2))

#define TEXT_START_X  ((BOX_Y+(BOX_H/2))/8)
#define TEXT_START_Y  ((BOX_X+24)/6)

#define ALL_SCC_LEDS_OFF  0x0
#define ALL_SCC_LEDS_ON   0xFFFFFFFF

#define BLINK_TIME  500U  /* ms */
#define SLEEP_TIME  300U  /* ms */

static const char glcd_text[] = "TOUCH!";
/*
 * \brief     Checks if the user is pressing the touchscreen central area.
 *
 * \parma[in] x   X position
 * \parma[in] y   Y position.
 *
 * \return    True if the user is pressing central area, otherwise false.
 */
bool is_cursor_in_center(int16_t x, int16_t y);
bool is_cursor_in_center(int16_t x, int16_t y)
{
    return (x > MIN_X && x < MAX_X && y > MIN_Y && y < MAX_Y);
}

int main(void)
{
    uint32_t count = ALL_SCC_LEDS_ON;
    struct mps2_touch_state_t ts_state;

    /* Initializes touchscreen and GLCD */
    (void)mps2_touch_init();
    mps2_glcd_init();

    /* Set GLCD background color */
    mps2_glcd_clear(MPS2_GLCD_WHITE);

    /* Draw rectangle in the touch active area */
    mps2_glcd_box_border(BOX_X-1, BOX_Y-1, BOX_W+1, BOX_H+1, MPS2_GLCD_BLUE);
    mps2_glcd_display_string(TEXT_START_X, TEXT_START_Y,
                             MPS2_GLCD_FONT_SIZE_6x8, glcd_text);

    mps2_set_leds(MPS2_SCC_LED, count);

    while(1) {
        /* If the MPS2 adapter for Arduino (AN502) is plugged in, replace
         * MPS2_FPGAIO_USERB0 and MPS2_FPGAIO_USERB1 by MPS2_SHIELD_USERB0 and
         * MPS2_SHIELD_USERB1 to use the buttons in the adapter instead of
         * USER buttons in the MPS2 board.
         */
        if(mps2_get_btn_state(MPS2_FPGAIO_USERB0) > 0) {
            /* Increment counter */
            count++;

            /* Log button pressed */
            mps2_led_on(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED0);
            mps2_led_off(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED1);

            /* Update SCC leds */
            mps2_set_leds(MPS2_SCC_LED, count);
        } else if (mps2_get_btn_state(MPS2_FPGAIO_USERB1) > 0) {
            /* Decrement counter */
            count--;

            /* Log button pressed */
            mps2_led_off(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED0);
            mps2_led_on(MPS2_FPGAIO_LED, MPS2_FPGAIO_USER_LED1);

            /* Update SCC leds */
            mps2_set_leds(MPS2_SCC_LED, count);
        }

        if(mps2_touch_get_state(&ts_state) == MPS2_TOUCH_ERR_OK) {
            if(ts_state.pressed &&
                is_cursor_in_center(ts_state.x, ts_state.y)) {

                /* Draw blue box with text in white*/
                mps2_glcd_box(BOX_X, BOX_Y,
                              BOX_W, BOX_H, MPS2_GLCD_BLUE);
                mps2_glcd_set_text_color(MPS2_GLCD_WHITE);
                mps2_glcd_set_back_color(MPS2_GLCD_BLUE);
                mps2_glcd_display_string(TEXT_START_X, TEXT_START_Y,
                                         MPS2_GLCD_FONT_SIZE_6x8, glcd_text);

                /* Blink SCC LEDs */
                mps2_set_leds(MPS2_SCC_LED, ALL_SCC_LEDS_OFF);
                mps2_sleepms(BLINK_TIME);
                mps2_set_leds(MPS2_SCC_LED, ALL_SCC_LEDS_ON);
                mps2_sleepms(BLINK_TIME);
                mps2_set_leds(MPS2_SCC_LED, count);

                /* Draw white box with text in black */
                mps2_glcd_box(BOX_X, BOX_Y,
                              BOX_W, BOX_H, MPS2_GLCD_WHITE);
                mps2_glcd_set_text_color(MPS2_GLCD_BLACK);
                mps2_glcd_set_back_color(MPS2_GLCD_WHITE);
                mps2_glcd_display_string(TEXT_START_X, TEXT_START_Y,
                                         MPS2_GLCD_FONT_SIZE_6x8, glcd_text);
            }
        }

        mps2_sleepms(SLEEP_TIME);
    }
}
