/*
 * Copyright (c) 2016-2017 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * \file apb_timer_drv.h
 * \brief Generic driver for APB Timers.
 */

#ifndef __APB_TIMER_DRV_H__
#define __APB_TIMER_DRV_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Default reload value */
#define APB_TIMER_DEFAULT_RELOAD  0xFFFFFFFF

/* APB timer device configuration structure */
struct apb_timer_dev_cfg_t {
    const uint32_t base;  /*!< Timer base address */
};

/* APB timer device data structure */
struct apb_timer_dev_data_t {
    uint32_t reload;  /*!< Timer Reload Value */
    uint32_t state;   /*!< Indicates if the timer
                           is initialized and enabled */
};

/* APB timer device structure */
struct apb_timer_dev_t {
    const struct apb_timer_dev_cfg_t* const cfg;  /*!< Timer configuration */
    struct apb_timer_dev_data_t* const data;      /*!< Timer data */
};

/* Timer enumeration types */
enum apb_timer_error_t {
    APB_TIMER_ERR_NONE = 0,     /*!< No error */
    APB_TIMER_ERR_NOT_INIT,     /*!< Error timer is not initialized */
};

/**
 * \brief Initializes a hardware timer.
 *
 * \param[in] dev       Timer configuration \ref apb_timer_dev_t
 * \param[in] load_val  Load timer in cycles - 0 assings timer to max value.
 */
void apb_timer_init(struct apb_timer_dev_t* dev, uint32_t load_val);

/**
 * \brief Verifies if a timer is enabled.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 *
 * \return 0 disabled - 1 enabled.
 */
uint32_t apb_timer_is_enabled(struct apb_timer_dev_t* dev);

/**
 * \brief Enables the given hardware timer.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 *
 * \return Returns error code as specified in \ref apb_timer_error_t
 */
enum apb_timer_error_t apb_timer_enable(struct apb_timer_dev_t* dev);

/**
 * \brief Disables the given hardware timer.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 */
void apb_timer_disable(struct apb_timer_dev_t* dev);

/**
 * \brief Reads timer value in us.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 *
 * \return Timer value or 0 if the timer is disabled.
 */
uint32_t apb_timer_read(struct apb_timer_dev_t* dev);

/**
 * \brief Enables timer Interrupt.
 *
 * \param[in] dev       Timer configuration \ref apb_timer_dev_t
 * \param[in] load_val  Load timer value in cycles.
 *
 * \return Returns error code as specified in \ref apb_timer_error_t
 */
enum apb_timer_error_t apb_timer_enable_interrupt(struct apb_timer_dev_t* dev,
                                                  uint32_t load_val);

/**
 * \brief Disables timer interrupt.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 */
void apb_timer_disable_interrupt(struct apb_timer_dev_t* dev);

/**
 * \brief Clears timer interrupt
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 */
void apb_timer_clear_interrupt(struct apb_timer_dev_t* dev);

/**
 * \brief Gets the interrupt status of the selected timer.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 *
 * \return Interrupt status of the selected singletimer.
 */
uint32_t apb_timer_get_status(struct apb_timer_dev_t* dev);

/**
 * \brief Gets the load value of the selected timer.
 *
 * \param[in] dev  Timer configuration \ref apb_timer_dev_t
 *
 * \return Reload value of the selected singletimer.
 */
uint32_t apb_timer_get_reload_value(struct apb_timer_dev_t* dev);

#ifdef __cplusplus
}
#endif
#endif /* __APB_TIMER_DRV_H__ */
